/*
 * False Flash application example for TIGCC,
 * by Rmi Denis-Courmont <rdenis@multimania.com>
 *
 * This is a very simple program to be used as a base in any freeware
 * TIGCC project for the TI-89 and/or TI-92+.
 */

#include "flashapp.h"
#include <alloc.h>
#include <dialogs.h>
#include <error.h>
#include <events.h>
#include <wingraph.h>

#define myAppName "TIINSLVR" // hooks the Numeric Solver
#define MY_APP_ID(p) EV_getAppID (myAppName)
#define ACM_MENU      0x500
#define ACM_UNINSTALL 0x501

static void myHandler (pFrame self, Event *evt); // events handler
static void QuitApp (AppID myID);
static HANDLE myToolbarBuild (void);

/*
 * Hook application frame. This is declared exactly as if it was a
 * frame from the TI Flash Studio SDK.
 */
FRAME(myApp, OO_SYSTEM_FRAME, 0, OO_APP_FLAGS, 4)
   ATTR (OO_APP_FLAGS, APP_INTERACTIVE)
   ATTR (OO_APP_NAME, "My application")
   ATTR (OO_APP_PROCESS_EVENT, myHandler)
   ATTR (OO_APP_DEFAULT_MENU, NULL) // this line is required!
   //ATTR (OO_APP_CAN_MOVE, alwaysFalseCallback)
   //ATTR (OO_APP_CAN_DELETE, alwaysFalseCallback)
/*
 * We don't have to implement OO_APP_CAN_MOVE and OO_APP_CAN_DELETE
 * as long as we hook a TIOS built-in application, as the Numeric
 * Solver, since these applications are never moved and/or deleted
 * (well, unless you update the AMS, of course).
 */
ENDFRAME

pFrame myApp_p = (pFrame) &myApp; // frame pointer

WINDOW myWindow; // main window
HANDLE myMenu = H_NULL; // current menu handle
unsigned char shouldQuit = 0; // abortion indicator

/*
 * Events handler. You will probably want to add a lot of things
 * there.
 */

static void myHandler (pFrame self, Event *evt)
{
   WINDOW *mainwnd = &myWindow;
   switch (evt->command)
   {
      case CM_INSTALL:
         /* initializing application settings... */
         break;

      case CM_START: // Starting application (creating main window)...
         if (!WinOpen (mainwnd, evt->info.startInfo.startRect,
                       WF_TTY | WF_DUP_SCR))
            shouldQuit = 1; // CM_FOCUS will call QuitApp()
         else
            WinClr (mainwnd);
         break;

      case CM_ACTIVATE: // Activating application...
         if (!shouldQuit)
         { // Building and starting dynamic menu...
            if (((myMenu = myToolbarBuild ()) == H_NULL)
             || (OO_SetAttr (self, OO_APP_DEFAULT_MENU_HANDLE,
                             (void *) (ULONG) myMenu) == FALSE))
            {
               shouldQuit =  2;
               return;
            }
            WinBeginPaint (mainwnd); // (WinBegin)
            WinActivate (mainwnd);
            WinStr (mainwnd, "Hello world!\n");
         }
         /*
          * As OO_APP_DEFAULT_MENU is NULL, EV_defaultHandler() does
          * not attempt to use a static menu. It simply registers the
          * menu whose running handle has been stored in
          * OO_APP_DEFAULT_MENU_HANDLE.
          */
         break;
            
      case CM_FOCUS: // Focus = 1st ability to abort if not enough mem
         if (shouldQuit)
            QuitApp (MY_APP_ID (myApp_p));
         break;
      
      /*case CM_UNFOCUS:
         break;*/
         
      case CM_DEACTIVATE: // Deactivating application...
         if (mainwnd->Next != NULL) // main window exists?
         {
            WinDeactivate (mainwnd);
            WinEndPaint (mainwnd); // (WinEnd)
         }
         myMenu = H_NULL;
         /* EV_defaultHandler() frees the running menu handle, unlocks
          * and frees the menu structure (because the menu was started
          * with flag MBF_HMENU), and sets OO_APP_DEFAULT_MENU_HANDLE
          * back to H_NULL.
          */
         break;
        
      case CM_QUIT: // Ending application (CM_ENDTASK)...
         if (mainwnd->Next != NULL)
         {
            WinClose (mainwnd);
            mainwnd->Next = NULL;
         }
         break;
      
      case CM_UNINSTALL:
         /* releasing application permanent resources... */
         break;

      case CM_WPAINT: // Repainting window...
      {
         WINDOW *dirtywnd = evt->info.paintInfo.window;
         DrawWinBorder (dirtywnd, &dirtywnd->Window);
         WinBackupToScr (dirtywnd);
      }
         break;

      case ACM_UNINSTALL: // a menu-driven event
         QuitApp (MY_APP_ID (myApp_p));
         ER_throwVar (1); // goes back to _main()
   }
   EV_defaultHandler ((EVENT *) evt);
}


/*
 *  Calls EV_quit (disable split-screen if required)
 */
static void QuitApp (AppID myID)
{
   Access_AMS_Global_Variables;
   MO_currentOptions ();

   /* If in full screen, just quit to switch to home */
   if (MO_option[MO_OPT_SPLIT_SCREEN] == D_MODE_SPLIT_FULL)
      EV_quit ();
   else
   {/* If in split screen, set the appropriate side to home.
     * AMS behavior is such that if home was already on one side,
     * it will become full screen. */
      AppID HomeID;
      HomeID = EV_getAppID ((UCHAR *) "TIHOME");

      if (MO_option[MO_OPT_SPLIT_1] == myID)
         MO_option[MO_OPT_SPLIT_1] = HomeID;
      if (MO_option[MO_OPT_SPLIT_2] == myID)
         MO_option[MO_OPT_SPLIT_2] = HomeID;

      MO_digestOptions (0);
   }
}

/*
 * This function dynamically builds the program menu. You may want to
 * use a static menu, instead. It returns the running menu handle on
 * succes, H_NULL on error.
 */
static HANDLE myToolbarBuild (void)
{           
   HANDLE menu, menuRun;  
   if ((menu = MenuNew (0, 100, 18)) == H_NULL)
      return H_NULL;
         
   if ((MenuAddText (menu, 0, "Menu", ACM_MENU, 0) != H_NULL)
    && (MenuAddText (menu, ACM_MENU, "Uninstall", ACM_UNINSTALL, 0) != H_NULL)
    && ((menuRun = MenuBegin (NULL, 0, 0, MBF_HMENU,  menu)) != H_NULL))
      return menuRun;

   HeapFree (menu);
   return H_NULL; // failure
}



/* Your event handler should throw an error to uninstall the program.
 * This will return to _main(), uninstall the hook frame and finally
 * unload the program from memory.
 */

#include <compat.h>

short _ti89;
short _ti92plus;

void HookDispError (char *msg)
{
   DlgMessage ("Hook installation failure", msg, 0, BT_CANCEL);
}

void _main (void)
{
   pFrame hook;
   AppID hookAppID;

   MO_currentOptions ();

   if ((hookAppID = EV_getAppID ((UCHAR *) myAppName)) == (AppID) -1)
      HookDispError ("Application not found"); /* This is unlikely to
       * happen while hooking a built-in application, but it is very
       * likely to happen with a Flash application. */
   else
   if (!AMS_2xx)
      HookDispError ("Unsupported AMS version"); /* Application
       * frames-oriented functions (OO_* rom calls) requires AMS 2.00
       * or higher. */
   else
   {  // quits the Numeric Solver if it is on the other side of the screen
      if ((ModeSettings->Split1App == hookAppID)
       || (ModeSettings->Split2App == hookAppID))
         QuitApp (hookAppID);

      // INSTALLING FRAME HOOK...
      if (OO_InstallAppHook (hookAppID, myApp_p, &hook) == FALSE)
         HookDispError ("Not enough memory");
      else
      {
         Event evt;
         /*
          * This is used to emulate CM_INSTALL and CM_UNINSTALL
          * events. This is only needed if your program expects and
          * handles these messages.
          */
         evt.command = CM_INSTALL;
         EV_sendEvent (hookAppID, (EVENT *) &evt);

         /*
          * This line automatically starts the newly installed false
          * Flash application (this is of course not mandatory):
          */
         EV_startApp (hookAppID, AP_START_CURRENT);
         
         TRY
            // RETURNING TO TIOS... (until an error is thrown)
            EV_eventLoop ();
         ONERR
         ENDTRY

/********************************************************************
 * IMPORTANT NOTE:                                                  *
 * Your program must not be active (not even on the "other side" of *
 * the screen) while throwing the exception!                        *
 ********************************************************************/

         evt.command = CM_UNINSTALL;
         EV_sendEvent (hookAppID, (EVENT *) &evt);

         // UNINSTALLING frame hook...
         OO_UninstallAppHook (hookAppID, hook);
         /*
          * We cannot do anything in case of error, and there is no
          * known reason why an error would occur here since both
          * <hookAppID> and <hook> are valid.
          */
      }
   }
}
