/*
 * Flash application-oriented TI AMS entry points and type definitions
 * for TIGCC and AMS version 2.00 or higher *only*
 * by Rmi Denis-Courmont <rdenis@multimania.com>
 *
 * Use at your own risk. This file is provided with NO WARRANTY of any sort.
 * Neither Rmi Denis-Courmont, nor Texas Instruments shall be liable for any
 * expectable or unexpectable, expected or unexpected, wanted or unwanted,
 * direct or indirect result of the use of this file.
 *
 * If you agree with the above disclaimer of warranty, unrem the following
 * #define . Then, thank you for using <flashapp.h> for TIGCC.
 *
 * This file may not be sold, bought, rent or lent. This may not be used in
 * non-freeware application without the prior digitally signed written consent
 * of Rmi Denis-Courmont.
 */
 //#define I_AGREE_WITH_THE_ABOVE_DISCLAIMER

#ifndef I_AGREE_WITH_THE_ABOVE_DISCLAIMER
# error You must agree with the disclaimer to use this file!
#else
# undef I_AGREE_WITH_THE_ABOVE_DISCLAIMER /* No longer needed. */
#endif

/*
 * Detailed informations on how to use the following functions is available in
 * the TI-89 and TI-92+ Programmer's Guide from Texas Instruments.
 */
#ifndef FLASHAPP_H
# define FLASHAPP_H

# include <asmtypes.h> // BOOL, ULONG
# include <events.h> // HANDLE - there are other way to get it, though

typedef ULONG pFrame; // 32-bits pointer
typedef HANDLE AppID;

/*
 * Various things having different names in TIGCC and TI Flash Studio
 */
# define WinBeginPaint WinBegin
# define WinEndPaint   WinEnd
# define MO_option     ((WORD *const) ModeSettings)


# define CM_NULL          CM_IDLE
# define CM_START         CM_STARTTASK
# define CM_QUIT          CM_ENDTASK
# define CM_RESTART       CM_START_CURRENT
# define CM_APD           0x709
# define CM_OFF           0x70A
# define CM_ON            0x70B // CM_DEFAULTS in TIGCC
# define CM_INSTALL       0x70C
# define CM_UNINSTALL     0x70D
# define CM_PACK          0x70E
# define CM_UNPACK        0x70F
# define CM_KEY_PRESS     CM_KEYPRESS
# define CM_CUT           CM_MENU_CUT
# define CM_COPY          CM_MENU_COPY
# define CM_PASTE         CM_MENU_PASTE
# define CM_PASTE_STRING  CM_STRING
# define CM_PASTE_HANDLE  CM_HSTRING
# define CM_DELETE        CM_DEL
# define CM_CLEAR         CM_CLR
# define CM_CLEAR_ALL     CM_MENU_CLEAR
# define CM_FIND          CM_MENU_FIND
# define CM_TOGGLE_INSERT CM_INSERT
# define CM_CURSOR_FLASH  CM_BLINK
# define CM_STO           CM_STORE
# define CM_RCL           CM_RECALL
# define CM_OPEN          CM_MENU_OPEN
# define CM_SAVE_AS       CM_MENU_SAVE
# define CM_NEW           CM_MENU_NEW
# define CM_FORMAT        CM_MENU_FORMAT
# define CM_ABOUT         CM_MENU_ABOUT
# undef CM_DEFAULTS
# define CM_DEFAULTS      0x782
# define CM_BACKGROUND    0x790
# define CM_OPEN_SYSDATA  CM_GEOMETRY


/*
 * Manifest constants to be used with MO_option
 */
enum
{
   MO_OPT_CURRENT_FOLDER,  /* do not change manually - use setFold() */
   MO_OPT_SPLIT_SCREEN,    /* see below */
   MO_OPT_NUMBER_OF_GRAPHS,/* see below (values are quite obvious anyway) */
   MO_OPT_GRAPH_TYPE_1,    /* see below */
   MO_OPT_GRAPH_TYPE_2,    /* same as MO_OPT_GRAPH_TYPE_1, for 2nd graph */
   MO_OPT_SPLIT_1,         /* Top/Left/Single application ID */
   MO_OPT_SPLIT_2,         /* Bottom/Right/Previous application ID */
   MO_OPT_SPLIT_RATIO,     /* see below */
   MO_OPT_ANGLE,           /* see below */
   MO_OPT_PRECISION,       /* see below */
   MO_OPT_FIX,             /* see below */  
   MO_OPT_NUMBER_FORMAT,   /* see below */
   MO_OPT_VECTOR_FORMAT,   /* see below */
   MO_OPT_COMPLEX_FORMAT,  /* see below */
   MO_OPT_PRETTY_PRINT,    /* see below */
   MO_OPT_BASE,            /* see below */
   MO_OPT_UNIT_SYSTEM,     /* see below */
   MO_OPT_UNIT_DEFAULTS    /* D_UNITS_DEFAULT=1 sets (how???) custom units */
/* The last 3 following options (language, item count (??) and units
 * are not officially supported by TIGCC
 */
};

enum /* values for MO_OPT_SPLIT_SCREEN */ {
   D_MODE_SPLIT_FULL=1, D_MODE_SPLIT_HORIZONTAL, D_MODE_SPLIT_VERTICAL };

enum /* values for MO_OPT_NUMBER_OF_GRAPHS */ {
   D_MODE_GRAPHS_1=1, D_MODE_GRAPHS_2 };

enum /* values for MO_OPT_GRAPH_TYPE_1 and MO_OPT_GRAPH_TYPE_2 */ {
   D_GRAPH_TYPE_FUNCTION=1, D_GRAPH_TYPE_PARAMETRIC, D_GRAPH_TYPE_POLAR,
   D_GRAPH_TYPE_SEQUENCE, D_GRAPH_TYPE_3D, D_GRAPH_TYPE_DIFF_EQUATIONS };

enum /* values for MO_OPT_SPLIT_RATIO */ {
   D_SPLIT_RATIO_1_1=1, D_SPLIT_RATIO_1_2, D_SPLIT_RATIO_2_1 };

enum /* values for MO_OPT_ANGLE */ {
   D_ANGLE_RAD=1, D_ANGLE_DEG };

enum /* values for MO_OPT_PRECISION */ {
   D_PREC_AUTO=1, D_PREC_RATIONAL, D_PREC_APPROX };

enum /* values for MO_OPT_FIX */ {
   D_PREC_FIX_0=1, D_PREC_FIX_1, D_PREC_FIX_2, D_PREC_FIX_3, D_PREC_FIX_4,
   D_PREC_FIX_5, D_PREC_FIX_6, D_PREC_FIX_7, D_PREC_FIX_8, D_PREC_FIX_9,
   D_PREC_FIX_10, D_PREC_FIX_11, D_PREC_FIX_12, D_PREC_FLOAT, D_PREC_FLOAT_1,
   D_PREC_FLOAT_2, D_PREC_FLOAT_3, D_PREC_FLOAT_4, D_PREC_FLOAT_5,
   D_PREC_FLOAT_6, D_PREC_FLOAT_7, D_PREC_FLOAT_8, D_PREC_FLOAT_9,
   D_PREC_FLOAT_10, D_PREC_FLOAT_11,  D_PREC_FLOAT_12 };

enum /* values for MO_OPT_NUMBER_FORMAT */ {
   D_EXP_FORMAT_NORMAL=1, D_EXP_FORMAT_SCI, D_EXP_FORMAT_ENG };

enum /* values for MO_OPT_VECTOR_FORMAT */ {
   D_VECT_RECT=1,  D_VECT_CYL,  D_VECT_SPH };

enum /* values for MO_OPT_COMPLEX_FORMAT */ {
   D_COMPLEX_OFF=1, D_COMPLEX_RECT, D_COMPLEX_POLAR };

enum /* values for MO_OPT_PRETTY_PRINT */ {
   D_ON=1, D_OFF };
 
enum /* values for MO_OPT_BASE */ {
   D_DEC=1, D_HEX, D_BIN };

enum /* values for MO_OPT_UNIT_SYSTEM */ {
   D_UNIT_SI=1, D_UNIT_US, D_UNIT_CUSTOM };
   
enum /* single (!) value for MO_OPT_UNIT_DEFAULTS
   I have no idea how this option works... */ {
   D_UNIT_DEFAULTS=1 };

/*
 * Event structure
 */
typedef UINT NOTIFY_FLAGS;

typedef struct __EventStruct
{
   UINT command;
   UINT sourceID;
   UINT side;
   UINT status;
   union
   {
      struct { DWORD info1; WORD info2; } eventInfo;
      struct { USHORT keyModifiers; USHORT keyCode; } keyInfo;
      struct { unsigned char *pasteString; } pasteInfo;
      struct { HANDLE pasteHandle; } pasteHandleInfo;
      struct { WINDOW *window; } paintInfo;
      struct { WIN_RECT *startRect; UCHAR startCode; } startInfo;
      struct { UINT menuItem; } menuInfo;
      struct { NOTIFY_FLAGS notifyFlags; } modeInfo;
   } info;
} Event, *PEvent;

/*
 * Application frames-related declaration
 */
typedef enum
{
   OO_RW=0,   // read-write access
   OO_RO=1,   // read-only access
   OO_SEQ=0,  // ??
   OO_KEYED=2 // ??
} OO_Flags;

typedef struct FrameHeaderStruct
{
   pFrame parent;
   pFrame prototype;
   OO_Flags flags;
   ULONG first;
   ULONG count;
} OO_Hdr;

typedef struct FrameAttributeStruct
{
   ULONG key;
   void *value;
} OO_Attr;

typedef struct FrameStruct
{
   OO_Hdr head;
   union
   {
      void *value[65000];
      OO_Attr pair[65000];
   } attr;
} Frame;

# define OO_HANDLE 0xFF000000u
# define OO_SYSTEM_FRAME OO_HANDLE

typedef void (* const OO_MethodPtr)(void);
# define STRING_FRAME( name, parent, prototype, first, count ) \
const OO_Hdr name = \
{ \
   (pFrame) parent, \
   (pFrame) prototype, \
   OO_RO | OO_SEQ, \
   first, \
   count \
}; \
static const char * const name##Attr[count] = \
{
# define FRAME( name, parent, proto, first, count ) \
const OO_Hdr name = \
{ \
   (pFrame) parent, \
   (pFrame) proto, \
   OO_RO | OO_KEYED, \
   first, \
   count \
}; \
static const OO_Attr name##Attr[count] = \
{
# define ATTR( selector, val ) { selector, (void *) (val) },
# define STRING_ATTR( sel, s ) {OO_FIRST_STRING + (sel), s},
# define ENDFRAME };

typedef struct AppExtensionStruct
{
   unsigned long name;
   unsigned long help;
   unsigned short index;
} APP_EXTENSION;

typedef struct AppExtEntryStruct
{
   void (*extension) (void);
   unsigned short flags;
} APP_EXT_ENTRY;

enum {
   APP_EXT_PROGRAM=0x0000, APP_EXT_FUNCTION=0x0001 };

enum /* constants for ACB.flags */{
   ACB_BUILTIN=0x1, ACB_INSTALLED=0x2, ACB_LOCALIZER=0x4, ACB_LOCK=0x8,
   ACB_JT_VERSION=0x10, ACB_SELECTED=0x20, ACB_COLLAPSE=0x800, ACB_BG0x1000,
   ACB_COMPRESS  =0x4000, ACB_DELETE    =0x8000 };

typedef struct AppHdrStruct
{
   ULONG magic;
   UCHAR name[8];
   BYTE  zeros[24];
   USHORT flags;
   ULONG dataLen;
   ULONG codeOffset;
   ULONG initDataOffset;
   ULONG initDataLen;
   ULONG optlen;
} AppHdr;

typedef struct ACBStruct
{
   USHORT flags;
   AppID myID;
   AppID next;
   AppID prev;
   ULONG publicstorage;
   AppHdr const *appHeader;
   BYTE const *certhdr;
   pFrame appData;
} ACB;

/*
 * MY_ACB and MY_APP_ID should only work with real Flash application
 * because they assume that the Application Control Block (ACB) is located
 * in memory just before the application frame pointer (frame_p), which is
 * totally false if the application frame is a hook.
# define MY_ACB(frame_p) ((ACB*)((BYTE*)&(frame_p)-OFFSETOF(ACB,appData)))
# define MY_APP_ID(frame_p) (MY_ACB(frame_p)->myID)
 */
# define OO_FIRST_STRING          0x1000
# define OO_FIRST_APP_STRING      OO_FIRST_STRING
# define OO_APPSTRING (OO_FIRST_STRING+OO_FIRST_APP_STRING)
# define OO_FIRST_APP_ATTR        0x10000

typedef enum /* for use with OO_APP_FLAGS attribute */ {
   APP_NONE=0x0,
   APP_INTERACTIVE=0x1,
   APP_CON=0x2,
   APP_ACCESS_SYSVARS=0x4,
   APP_BACKGROUND=0x8
} APP_Flags;


# define TIGCC_romcall( return_type, parameters, hex_entry ) \
         _rom_call(return_type, parameters, hex_entry)

// rom call 1018
# define OO_CondGetAttr TIGCC_romcall(BOOL,(pFrame,ULONG,void**),3FA)
// rom call 1019
# define OO_Deref TIGCC_romcall(Frame *,(pFrame),3FB)
// rom call 1020
# define OO_GetAppAttr TIGCC_romcall(void *,(AppID,ULONG),3FC)
// rom call 1021
# define OO_GetAttr TIGCC_romcall(void *,(pFrame,ULONG),3FD)
// rom call 1022
# define OO_HasAttr TIGCC_romcall(BOOL,(pFrame,ULONG),3FE)
// rom call 1023
# define OO_New TIGCC_romcall(void *,(pFrame),3FF)
// rom call 1024
# define OO_SetAppAttr TIGCC_romcall(BOOL,(AppID,ULONG,void *),400)
// rom call 1025
# define OO_SetAttr TIGCC_romcall(BOOL,(pFrame,ULONG,void *),401)
// rom call 1026
# define OO_NextACB TIGCC_romcall(AppID,(AppID),402)
// rom call 1027
# define OO_PrevACB TIGCC_romcall(AppID,(AppID),403)
// rom call 1028
# define OO_InstallSystemHook TIGCC_romcall(BOOL,(pFrame, pFrame *),404)
// rom call 1029
# define OO_UninstallSystemHook TIGCC_romcall(BOOL,(pFrame),405)
// rom call 1030
# define OO_AppNameToACB TIGCC_romcall(AppID,(UCHAR const *, BOOL),406)

// rom call 1059
# define OO_Destroy TIGCC_romcall(pFrame,(pFrame),423)
// rom call 1060 is not documented and unknown to me

// rom call 1061
# define OO_appGetPublicStorage TIGCC_romcall(ULONG,(void),425)
// rom call 1062
# define OO_appIsMarkedDelete TIGCC_romcall(BOOL,(AppID),426)
// rom call 1063
# define OO_appMarkDelete TIGCC_romcall(void,(AppID),427)
// rom call 1064 is not documented and unknown to me

// rom call 1065
# define OO_appSetPublicStorage TIGCC_romcall(void,(ULONG),429)

// rom call 1166
# define EV_quit TIGCC_romcall(void,(void),48E)
// rom call 1167 is not documented and unknown to me

// rom call 1168
# define OO_InstallAppHook TIGCC_romcall(BOOL,(AppID,pFrame,pFrame *),490)
// rom call 1169
# define OO_UninstallAppHook TIGCC_romcall(BOOL,(AppID,pFrame),491)
// rom call 1170
# define OO_InstallAppHookByName \
   TIGCC_romcall(BOOL,(UCHAR const *,pFrame,pFrame *),492)
// rom call 1171
# define OO_UninstallAppHookByName \
   TIGCC_romcall(BOOL,(UCHAR const *,pFrame),493)

// rom call 1269
# define OO_DestroyAll TIGCC_romcall(pFrame,(pFrame),4F5)

enum
{
   OO_APP_FLAGS=1,
# define GetAppFlags( AppID ) (APP_Flags) OO_GetAppAttr (AppID, OO_APP_FLAGS)
# define SetAppFlags( AppID, flags ) \
   OO_SetAppAttr(AppID, OO_APP_FLAGS, (void *) flags)

   OO_APP_NAME,
# define GetAppName( AppID ) (UCHAR *) OO_GetAppAttr (AppID, OO_APP_NAME)
# define SetAppName(AppID, name) \
   OO_SetAppAttr (AppID, OO_APP_NAME, (void *) name)

   OO_APP_TOK_NAME,
# define GetAppTokName( AppID ) (UCHAR *) OO_GetAppAttr (AppID, OO_APP_TOK_NAME)
# define SetAppTokName( AppID, value ) \
   OO_SetAppAttr (AppID, OO_APP_TOK_NAME, (void *) value)

   OO_APP_PROCESS_EVENT,
# define GetAppProcessEvent( AppID ) \
   (void (* const)(pFrame, PEvent)) (OO_GetAttr (AppID, OO_APP_PROCESS_EVENT))
# define SetAppProcessEvent( AppID, EventHandler ) \
   OO_SetAttr (AppID, OO_APP_PROCESS_EVENT, (void *) EventHandler)
# define AppProcessEvent( AppID, evt ) (GetAppProcessEvent (AppID)) (AppID, evt)

   OO_APP_DEFAULT_MENU,
# define GetAppDefaultMenu( AppID ) \
   (void *) OO_GetAppAttr (AppID, OO_APP_DEFAULT_MENU)
# define SetAppDefaultMenu( AppID, menu ) \
   OO_SetAppAttr (AppID, OO_APP_DEFAULT_MENU, (void *) menu)

   OO_APP_DEFAULT_MENU_HANDLE,
# define GetAppDefaultMenuHandle( AppID ) \
   (HANDLE) OO_GetAppAttr (AppID, OO_APP_DEFAULT_MENU_HANDLE)
# define SetAppDefaultMenuHandle( AppID, handle ) \
   OO_SetAppAttr (AppID, OO_APP_DEFAULT_MENU_HANDLE, (void *) (ULONG) handle)
/* Cast to unsigned long before pointer avoid "Cast to pointer from integer
 * of different size" warning generated by TIGCC compiler */

   OO_APP_EXT_COUNT,
# define GetAppExtCount( AppID ) (long) OO_GetAppAttr (AppID, OO_APP_EXT_COUNT)
# define SetAppExtCount(AppID, num) \
   OO_SetAppAttr (AppID, OO_APP_EXT_COUNT, (void *) num)

   OO_APP_EXTENSIONS,
# define GetAppExtensions( AppID ) \
  (APP_EXTENSION const *) OO_GetAppAttr (AppID, OO_APP_EXTENSIONS)
# define SetAppExtensions( AppID, ext ) \
   OO_SetAppAttr (AppID, OO_APP_EXTENSIONS, (void *) ext)

   OO_APP_EXT_ENTRIES,
# define GetAppExtEntries( AppID ) \
   (APP_EXT_ENTRY const *) OO_GetAppAttr (AppID, OO_APP_EXT_ENTRIES)
# define SetAppExtEntries(AppID, entries) \
   OO_SetAppAttr (AppID, OO_APP_EXT_ENTRIES, (void *) entries)

   OO_APP_LOCALIZE,
# define GetAppLocalize( AppID ) \
   (BOOL (* const)(AppID, UCHAR const *)) (OO_GetAppAttr (AppID, \
                                                          OO_APP_LOCALIZE))
# define SetAppLocalize( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_LOCALIZE, (void *) entry)
# define AppLocalize( AppID, a ) (GetAppLocalize (AppID)) (AppID, a)

   OO_APP_UNLOCALIZE,
# define GetAppUnlocalize( AppID ) \
   (BOOL (* const)(AppID)) (OO_GetAppAttr (AppID, OO_APP_UNLOCALIZE))
# define SetAppUnlocalize( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_UNLOCALIZE, (void *) entry)
# define AppUnlocalize( AppID ) (GetAppUnlocalize (AppID)) (AppID)

   OO_APP_CAN_DELETE,
# define GetAppCanDelete( AppID ) \
   (BOOL (* const)(AppID)) (OO_GetAppAttr (AppID, OO_APP_CAN_DELETE))
# define SetAppCanDelete( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_CAN_DELETE, (void *) entry)
# define AppCanDelete( AppID ) (GetAppCanDelete (AppID)) (AppID)

   OO_APP_CAN_MOVE,
# define GetAppCanMove( AppID ) \
   (BOOL (* const)(AppID)) (OO_GetAppAttr (AppID, OO_APP_CAN_MOVE))
# define SetAppCanMove( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_CAN_MOVE, (void *) entry)
# define AppCanMove( AppID ) (GetAppCanMove (AppID)) (AppID)

   OO_APP_VIEWER,
# define GetAppViewer( AppID ) \
   (BOOL(*const)(AppID, BYTE *, WINDOW *, HSYM)) (OO_GetAppAttr (AppID, \
                                                                 OO_APP_VIEWER))
# define SetAppViewer( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_VIEWER, (void *) entry)
# define AppViewer( AppID, a, b, c) (GetAppViewer (AppID)) (AppID, a,b,c)

   OO_APP_ICON,
# define GetAppIcon( AppID ) (BITMAP *) OO_GetAppAttr (AppID, OO_APP_ICON)
# define SetAppIcon(AppID, icon) OO_SetAppAttr (AppID, OO_APP_ICON,(void *)icon)

   OO_APP_EXT_HELP,
# define GetAppExtHelp( AppID ) \
   (void (* const) (AppID, USHORT)) OO_GetAppAttr (AppID, OO_APP_EXT_HELP)
# define SetAppLocalize( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_LOCALIZE, (void *) entry)
# define AppExtHelp( AppID, num ) (GetAppExtHelp (AppID)) (AppID, num)

   OO_APP_NOTICE_INSTALL,
# define GetAppNoticeInstall( AppID ) \
   (void (* const)(AppID, ACB const *)) (OO_GetAppAttr (AppID, \
                                                        OO_APP_NOTICE_INSTALL))
# define SetAppNoticeInstall( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_NOTICE_INSTALL, (void *) entry)
# define AppNoticeInstall( AppID, acb ) (GetAppNoticeInstall (AppID))(AppID,acb)

   OO_APP_ABOUT,
# define GetAppAbout( AppID ) \
   (char const * (* const)(AppID)) (OO_GetAppAttr (AppID, OO_APP_ABOUT))
# define SetAppAbout( AppID, entry ) \
   OO_SetAppAttr (AppID, OO_APP_ABOUT, (void *) entry)
# define AppAbout( AppID ) (GetAppBout (AppID)) (AppID)

/* *** Unknown use ***
   OO_SFONT=768,
# define OO_APP_SFONT OO_SFONT
# define Getsfont( AppID ) (SF_CHAR *) OO_GetAttr (AppID, OO_SFONT)
# define Setsfont(AppID, font) OO_SetAttr (AppID, OO_SFONT, (void *) font)
# define GetAppSFont( AppID ) (SF_CHAR *) OO_GetAppAttr (AppID, OO_APP_SFONT)
# define SetAppSFont(AppID, font) \
   OO_SetAppAttr (AppID, OO_APP_SFONT, (void *) font)

   OO_LFONT,
# define OO_APP_LFONT OO_LFONT
# define Getlfont( AppID ) (LF_CHAR *) OO_GetAttr (AppID, OO_LFONT)
# define Setlfont( AppID, font ) OO_SetAttr (AppID, OO_LFONT, (void *) font)
# define GetAppLFont( AppID ) (LF_CHAR *) OO_GetAppAttr (AppID, OO_APP_LFONT)
# define SetAppLFont(AppID, font) \
   OO_SetAppAttr (AppID, OO_APP_LFONT, (void *) font)

   OO_HFONT,
# define OO_APP_HFONT OO_HFONT
# define Gethfont( AppID ) (HF_CHAR *) OO_GetAttr (AppID, OO_HFONT)
# define Sethfont(AppID, font) OO_SetAttr (AppID, OO_HFONT, (void *) font)
# define GetAppHFont( AppID ) (HF_CHAR *) OO_GetAppAttr (AppID, OO_APP_HFONT)
# define SetAppHFont(AppID, font) \
   OO_SetAppAttr (AppID, OO_APP_HFONT, (void *) font)
*/
   OO_LANGUAGE=784,
# define GetLanguage( AppID ) (WORD) OO_GetAttr (AppID, OO_LANGUAGE)
# define SetLanguage( AppID, lng ) OO_SetAttr (AppID, OO_LANGUAGE, (void *) lng)

   OO_DATE_FORMAT,
# define GetDateFormat( AppID ) (char const *) OO_GetAttr (AppID,OO_DATE_FORMAT)
# define SetDateFormat( AppID, format_str ) \
   OO_SetAttr (AppID, OO_DATE_FORMAT, (void *) format_str)

/*
   OO_BUILTIN_HELP,
# define BuiltinHelp(AppID, a) \
   ((void (* const)(pFrame, USHORT)) OO_GetAttr(AppID, 786))(AppID, a)

   OO_KTLIST=800,
# define GetKTList( AppID ) \
   (keytag const *) OO_GetAttr(AppID, 800)
# define SetKTList(AppID, value) \
   OO_SetAttr(AppID, 800,(void *)value)

   OO_CAT_TABLE,
# define GetCAT_table( AppID ) \
   (CATALOG const *) OO_GetAttr(AppID, 801)
# define SetCAT_table(AppID, value) \
   OO_SetAttr(AppID, 801,(void *)value)

   OO_CAT_INDEX,
# define GetCAT_index( AppID ) \
   (short const *) OO_GetAttr(AppID, 802)
# define SetCAT_index(AppID, value) \
   OO_SetAttr(AppID, 802,(void *)value)

   OO_CAT_COUNT,
# define GetCAT_count( AppID ) \
   (short) OO_GetAttr(AppID, 803)
# define SetCAT_count(AppID, value) \
   OO_SetAttr(AppID, 803,(void *)value)

   OO_CHAR_MENU=816,
# define GetCharMenu( AppID ) \
   (const MENU *) OO_GetAttr(AppID, 816)
# define SetCharMenu(AppID, value) \
   OO_SetAttr(AppID, 816,(void *)value)

   OO_CHAR_HANDLER,
# define CharHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 817))(AppID)

   OO_APPS_HANDLER,
# define AppsHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 818))(AppID)

   OO_FLASH_APPS_HANDLER,
# define FlashAppsHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 819))(AppID)

   OO_MATH_HANDLER,
# define MathHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 820))(AppID)

   OO_MEM_HANDLER,
# define MemHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 821))(AppID)

   OO_STO_HANDLER,
# define StoHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 822))(AppID)

   OO_QUIT_HANDLER
# define QuitHandler( AppID ) \
   ((void (* const)(pFrame)) OO_GetAttr(AppID, 823))(AppID)
*/
};

// rom call 54, improperly implemented in <menus.h> from TIGCC
# include <menus.h> // make sure <menus.h> will not be included further
# undef MenuBegin
# define MenuBegin \
   TIGCC_romcall (HANDLE, (const void *, short, short, SINT, ...), 36)
enum /* to be ORed in the forth MenuBegin parameter */ {
   MBF_REDEF=0x02, MBF_SYS_ALLOC=0x04, MBF_MAX_MENU_WIDTH=0x08,
   MBF_STRIKEOUT=0x10, MBF_HMENU=0x20, MBF_NO_DRAWTOP=0x40 };

// rom call 659
/*
# include <system.h>
# undef XR_stringPtr
char * tiamsapi_659(ULONG);
#define XR_stringPtr TIGCC_romcall (char *, (ULONG), 293)
*/

# define Access_AMS_Global_Variables { }
/* TIGCC needn't a macro to give access to global variables */

#endif /* ifndef FLASHAPP_H */

